<?xml version="1.0" encoding="utf-8"?>
<!--Script to validate discretion documents against test suite configuration-->
<!--$Date: 2001/08/21 20:16:39 $(UTC)-->

<!--not documented yet-->

<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                version="1.0">

<xsl:variable name="configuration" 
           select="document(doc/@configuration-href,doc/@configuration-href)"/>
<xsl:variable name="doc" select="/doc"/>

<xsl:output indent="yes"/>

<xsl:template match="/">
  <problems>
    <xsl:if test="not($configuration)">
      <xsl:variable name="err">
        <xsl:text>Configuration file '</xsl:text>
        <xsl:value-of select="/doc/@configuration-href"/>
        <xsl:text>' not found.</xsl:text>
      </xsl:variable>
      <problem><xsl:copy-of select="$err"/></problem>
      <xsl:message terminate="yes"><xsl:copy-of select="$err"/></xsl:message>
    </xsl:if>
    <xsl:for-each select="/doc/verbiage/group-intro">
      <xsl:variable name="status" select="@status"/>
      <xsl:for-each select="$configuration">
        <xsl:if test="name(id($status)) != 'discretion-status'">
          <problem value="{$status}">bad discretion status value</problem>
        </xsl:if>
      </xsl:for-each>
    </xsl:for-each>
    <xsl:for-each select="$configuration/suite/
                          discretion-statuses/discretion-status/@id">
      <xsl:if test="not($doc/verbiage/group-intro/@status=.)">
        <problem value="{.}">missing group-intro definition <xsl:text/>
                  <xsl:text/>for status value</problem>
      </xsl:if>
    </xsl:for-each>
    <xsl:for-each select="/doc/items/item">
      <xsl:variable name="citation-types" select="spec-citation/@type"/>
      <xsl:for-each select="$configuration">
        <xsl:if test="not(id($citation-types)/@method='human'
                      and id($citation-types)/@method='machine')">
          <problem value="{$citation-types/ancestor::item/@id}">
            <xsl:text>each item must have at least one human </xsl:text>
            <xsl:text>and one machine citation</xsl:text>
          </problem>
        </xsl:if>
      </xsl:for-each>
    </xsl:for-each>
    <xsl:for-each select="/doc/items/item/spec-citation">
      <xsl:variable name="spec" select="@spec"/>
      <xsl:for-each select="$configuration">
        
        <xsl:if test="name(id($spec)) != 'cite-spec'">
          <problem value="{$spec}">unexpected specification cited</problem>
        </xsl:if>
      </xsl:for-each>
    </xsl:for-each>
  </problems>
</xsl:template>

</xsl:stylesheet>
